import hab
from Qt import QtWidgets

from .. import utils
from .alias_icon_button import AliasIconButton


class AliasButtonGrid(QtWidgets.QWidget):
    """Create a grid layout to hold buttons that are used to launch alias
    applications.

    Args:
        resolver (hab.Resolver): The resolver to change verbosity settings on.
        button_wrap_length (int) Inidicates the number of buttons per column/row.
        button_layout (int) Sets the button layout to be either a horizontal focus
            or a vertical focus.
        verbosity (int): Change the verbosity setting to this value. If None is passed,
            all results are be shown without any filtering.
        uri (string, optional) The project uri that specifies the button aliases.
        button_cls (QToolButton, optional): The button class that populates
            the grid.
        parent (Qt.QtWidgets.QWidget, optional): Define a parent for this widget.
    """

    def __init__(
        self,
        resolver,
        button_wrap_length,
        button_layout,
        verbosity,
        uri=None,
        button_cls=AliasIconButton,
        parent=None,
    ):
        super().__init__(parent)
        self.resolver = resolver
        self.button_wrap_length = button_wrap_length
        self.button_layout = button_layout
        self.verbosity = verbosity
        self.uri = uri
        self.button_cls = button_cls

        self.grid_layout = QtWidgets.QGridLayout(self)
        self.grid_layout.setContentsMargins(0, 0, 0, 0)
        self.setLayout(self.grid_layout)

    def refresh(self):
        self.clear()
        if self.uri is None:
            return
        cfg = self.resolver.resolve(self.uri)
        with hab.utils.verbosity_filter(self.resolver, self.verbosity):
            alias_list = list(cfg.aliases.keys())
            # So buttons show up in alphabetical order
            alias_list.sort()
        button_coords = utils.make_button_coords(
            alias_list, self.button_wrap_length, self.button_layout
        )
        for button_name, button_coord in button_coords.items():
            button = self.button_cls(cfg, button_name)
            self.grid_layout.addWidget(button, button_coord[0], button_coord[1])
        self.spacer_item = QtWidgets.QSpacerItem(
            20, 0, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding
        )
        self.grid_layout.addItem(self.spacer_item, self.grid_layout.rowCount(), 0, 1, 1)

    def clear(self):
        while self.grid_layout.count():
            item = self.grid_layout.takeAt(0)
            widget = item.widget()
            if widget is not None:
                widget.deleteLater()
            else:
                self.grid_layout.removeWidget(widget)
                self.grid_layout.removeItem(item)



class BaseInit:
    """Base class that can be called by `hab_gui.utils.entry_point_init` to
    customize some part of hab_gui based on site configuration.

    You don't need to subclass this class, just make sure your class conforms to it.
    """

    def __init__(self, resolver, cmd, cli_args=None, **kwargs):
        super().__init__()
        self.resolver = resolver
        self.cmd = cmd
        self.cli_args = cli_args
        self.kwargs = kwargs



from Qt import QtGui, QtWidgets


class SplashScreen(QtWidgets.QSplashScreen):
    """A widget that provides a QSplashScreen that can be used for long
    load times.  This subclass can use both static and animated images.

    Args:
        path_to_image (string): The full filepath to an image.
    """

    def __init__(self, path_to_image: str):
        # QMovie can be used for both animated and static images
        self.movie = QtGui.QMovie(path_to_image)
        self.movie.jumpToFrame(0)
        pixmap = QtGui.QPixmap(self.movie.frameRect().size())
        QtWidgets.QSplashScreen.__init__(self, pixmap)
        self.movie.frameChanged.connect(self.repaint)

    def showEvent(self, event):  # noqa: N802
        self.movie.start()

    def hideEvent(self, event):  # noqa: N802
        self.movie.stop()

    def paintEvent(self, event):  # noqa: N802
        painter = QtGui.QPainter(self)
        pixmap = self.movie.currentPixmap()
        self.setMask(pixmap.mask())
        painter.drawPixmap(0, 0, pixmap)



import logging

import click
from hab.cli import UriArgument, UriHelpClass

from . import utils
from .widgets.splash_screen import SplashScreen

logger = logging.getLogger(__name__)


def get_application(settings=None, splash=True, **kwargs):
    """Returns the QApplication instance and SplashScreen, creating it if required.

    Args:
        settings (hab.cli.SharedSettings, optional): If settings is passed, then
            the `hab_gui_init` entry point is processed.
        splash (bool, optional): If enabled and the `splash_screen` property of
            the site config contains an image path. A SplashScreen is created
            shown and returned.
        **kwargs: Any other kwargs are passed to `cli_args` of
            `hab_gui.utils.entry_point_init`.

    Returns:
        app, splash: The first item is the QApplication instance. The second is
            the splash screen instance that was shown, or None.
    """
    from Qt.QtWidgets import QApplication

    global app

    if settings:
        utils.entry_point_init(settings.resolver, "launch", cli_args=kwargs)

    # Get the existing app if possible
    app = QApplication.instance()
    _splash = None
    if not app:
        # Otherwise create a new QApplication instance
        app = QApplication([])
        # Attempt to show a splash screen in case it takes a little while to
        # fully process the hab configuration
        if splash:
            splash_image = utils.get_splash_image(settings.resolver)
            if splash_image:
                _splash = SplashScreen(splash_image)
                _splash.show()

    return app, _splash


@click.group()
@click.pass_context
def gui(ctx):
    """Run hab commands using gui mode"""


@gui.command(cls=UriHelpClass)
@click.option(
    "-v",
    "--verbose",
    "verbosity",
    count=True,
    help="Show increasingly detailed output. Can be used up to 3 times.",
)
@click.argument("uri", cls=UriArgument, required=False)
@click.pass_obj
def launch(settings, verbosity, uri):
    """Show a gui letting the user launch applications."""
    from .windows.alias_launch_window import AliasLaunchWindow

    if isinstance(uri, click.UsageError):
        # Launch doesn't require a URI, but if its not passed a UsageError
        # is returned by UriArgument. Convert that to None.
        uri = None

    app, splash = get_application(settings, uri=uri, verbosity=verbosity)

    settings.resolver._verbosity_target = "hab-gui"

    window = AliasLaunchWindow(settings.resolver, uri=uri, verbosity=verbosity)
    window.show()
    if splash:
        splash.finish(window)

    app.exec_()


@gui.command()
@click.argument("uri", required=False)
@click.pass_obj
def set_uri(settings, uri):
    """Allows for saving a local URI default by passing
    a URI argument.  If no argument is passed uri-set
    will prompt you to enter and argument."""

    settings.log_context(uri)

    from Qt.QtWidgets import QInputDialog, QMessageBox

    # Create the QApplication, app.exec_ currently does not need called due
    # to this only using QInputDialog and QMessageBox.
    _ = get_application(settings, uri=uri, splash=False)

    if uri is not None:
        # If the uri was passed, no need to ask the user
        settings.resolver.user_prefs().uri = uri
        QMessageBox.information(
            None, "Hab URI default set", f"The hab URI default was set to {uri}"
        )
        return

    # Otherwise ask the user what uri to use.
    # TODO: Use fancy hab-gui widgets configured by site
    current_uri = settings.resolver.user_prefs().uri
    uris = list(settings.resolver.dump_forest(settings.resolver.configs, indent=""))
    if current_uri and current_uri not in uris:
        uris.append(current_uri)
    uris.sort(key=str.casefold)

    current_index = -1
    if current_uri:
        current_index = uris.index(current_uri)

    uri, ok = QInputDialog.getItem(
        None, "Set hab URI", "Set default hab URI to:", uris, current=current_index
    )
    if ok:
        settings.resolver.user_prefs().uri = uri



import logging

from Qt import QtWidgets

from .. import utils

logger = logging.getLogger(__name__)


class PinnedUriButton(QtWidgets.QToolButton):
    """A widget that gives users quick access to URI's used regularly. When the
    user clicks on this tool button, it opens a menu allowing the user to choose
    from previously saved URI's or add/remove a URI to saved user_prefs.

    Args:
        resolver (hab.Resolver): The resolver used for settings.
        uri_widget (QWidget): The URIComboBox like widget used to get/set the
            current URI from.
        verbosity (int): Pass along a verbosity value for filtering of URIs
        parent (Qt.QtWidgets.QWidget, optional): Define a parent for this widget.
    """

    _text_main = "\U0001F4CC"
    _text_pin_selected = "Pin selected URI"
    _text_remove_uri = "Remove pin"

    def __init__(self, resolver, uri_widget, verbosity=0, parent=None):
        super().__init__(parent)
        self.resolver = resolver
        self.uri_widget = uri_widget
        self.verbosity = verbosity

        self.setToolTip("Select and manage quick access to commonly used URI's.")
        self.setText(self._text_main)
        self.setIcon(utils.Paths.icon("pin-outline.svg"))
        self.setPopupMode(self.InstantPopup)
        self.refresh()

    def add_uri(self, uri):
        """Add this uri to `self.uris()` and save that change to user_prefs."""
        uris = self.uris()
        uris.add(uri)
        self.set_uris(uris)

    def menu_triggered(self, action):
        """Handles all actions selected by the user in the menu."""
        uri = action.text()
        tag = action.data()
        if tag == "pin":
            # This option uses the URI from the uri_widget not the action
            uri = self.uri_widget.uri()
            self.add_uri(uri)
            self.refresh()
        elif tag == "choose":
            self.uri_widget.set_uri(uri)
        elif tag == "remove":
            # Remove this URI from user_prefs
            uris = self.uris()
            if uri in uris:
                uris.remove(uri)
                self.set_uris(uris)
            self.refresh()

    def refresh(self):
        """Rebuilds the menu shown when a user clicks on the button."""
        menu = QtWidgets.QMenu(self)
        menu.triggered.connect(self.menu_triggered)

        # Add management actions and menus
        act = menu.addAction(self._text_pin_selected)
        act.setIcon(utils.Paths.icon("pin-outline.svg"))
        act.setData("pin")
        remove_menu = menu.addMenu(self._text_remove_uri)
        remove_menu.setIcon(utils.Paths.icon("pin-off-outline.svg"))
        menu.addSeparator()

        # Add existing pinned URI's to both menus
        for uri in sorted(self.uris(), key=str.casefold):
            # Selects this URI in self.uri_widget
            act = menu.addAction(uri)
            act.setData("choose")

            # Removes the URI from saved user_prefs
            act = remove_menu.addAction(uri)
            act.setData("remove")

        self.setMenu(menu)

    def uris(self):
        """Returns the pinned_uris saved in preferences as a set. It will only do
        that if prefs are enabled. Returns `set()` otherwise. This will call load
        to ensure the preference file has been loaded.
        """
        prefs = self.resolver.user_prefs()
        if prefs.enabled:
            # Ensure the preferences are loaded.
            prefs.load()
            return set(prefs.get("pinned_uris", []))
        return set()

    def set_uris(self, uris):
        """Saves URIS to pinned_uris in user_prefs. It will only do that if prefs
        are enabled. This will call load to ensure the preference file has been loaded.
        """
        prefs = self.resolver.user_prefs()
        if prefs.enabled:
            # Ensure the preferences are loaded.
            prefs.load()

            prefs["pinned_uris"] = sorted(uris, key=str.casefold)
            prefs.save()
            logger.debug(f"Pinned URI's saved {prefs.filename}")



import logging

from Qt import QtWidgets

from .. import utils

logger = logging.getLogger(__name__)


class MenuButton(QtWidgets.QToolButton):
    """A button that gives the user access to a menu for the hab launcher.

    Args:
        resolver (hab.Resolver): The resolver used for settings.
        hab_widget (QWidget): The URI widget menu operations are performed on.
            This likely is also the parent, but may not be.
        verbosity (int): Pass along a verbosity value for filtering of URIs
        parent (Qt.QtWidgets.QWidget, optional): Define a parent for this widget.
    """

    def __init__(self, resolver, hab_widget, verbosity=0, parent=None):
        super().__init__(parent=parent)
        self.hab_widget = hab_widget
        self.resolver = resolver
        self.verbosity = verbosity

        self.setText("Menu")
        self.setIcon(utils.Paths.icon("menu.svg"))
        self.setPopupMode(self.InstantPopup)
        self.refresh()

    def populate_menu(self, menu):
        """Builds the menu by adding QActions with connected signals."""
        act = menu.addAction("Refresh Hab Config")
        act.setObjectName("refresh_hab_cfg")
        act.setIcon(utils.Paths.icon("refresh.svg"))
        act.triggered.connect(self.refresh_hab_config)

    def refresh(self):
        """Rebuilds the menu shown when a user clicks on the button.
        See `populate_menu` for how the menu is populated.
        """
        menu = QtWidgets.QMenu(self)

        # Add actions and menus
        self.populate_menu(menu)

        self.setMenu(menu)

    def refresh_hab_config(self):
        """Reset the refresh_timer if active and calls refresh_cache."""
        running = self.hab_widget.refresh_timer.isActive()
        if running:
            self.hab_widget.refresh_timer.stop()

        self.hab_widget.refresh_cache()

        if running:
            self.hab_widget.refresh_timer.start()



import logging
import os

from Qt import QtCore, QtGui

from .alias_button import AliasButton

logger = logging.getLogger(__name__)


class AliasIconButton(AliasButton):
    """Create a AliasButton that also shows a icon for each alias.

    Args:
        cfg (hab.parsers.flat_config.FlatConfig): The config object which contains
        URI related data.
        alias_name (str): The alias name passed from AliasButtonGrid.
        parent (Qt.QtWidgets.QWidget, optional): Define a parent for this widget.
    """

    button_pressed = QtCore.Signal(str)

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.setToolButtonStyle(QtCore.Qt.ToolButtonTextBesideIcon)

    def refresh(self):
        alias = self.alias_dict[self.alias_name]
        icon = QtGui.QIcon()
        icon_path = alias.get("icon", "")
        if os.path.exists(icon_path):
            icon.addPixmap(QtGui.QPixmap(icon_path))
        elif icon_path:
            logger.debug(
                f"The specified icon file {icon_path} does not exist for {self.alias_dict}"
            )
        self.setIcon(icon)

        super().refresh()



import logging

from Qt import QtWidgets

from .. import utils

logger = logging.getLogger(__name__)


class RefreshButton(QtWidgets.QToolButton):
    """Create a QToolButton that forces hab to re-cache its data and update hab-gui.

    Args:
        resolver (hab.Resolver): The hab resolver to operate on.
        verbosity (int): Pass along a verbosity value for filtering of URIs
        parent (Qt.QtWidgets.QWidget, optional): Define a parent for this widget.
    """

    def __init__(self, resolver, verbosity=0, parent=None):
        super().__init__(parent)
        self.resolver = resolver
        self.verbosity = verbosity

        self.setToolTip("Refresh hab to ensure you have the latest configuration.")
        self.setText("Refresh")
        self.setIcon(utils.Paths.icon("refresh.svg"))



import logging

import hab
from Qt import QtCore, QtWidgets

from .. import utils

logger = logging.getLogger(__name__)


class AliasLaunchWindow(QtWidgets.QMainWindow):
    """Create a window that can launch applications relevant to a project URI

    Contains a custom QComboBox (URIComboBox) and a QGridlayout (AliasButtonGrid)
    The ComboBox is populated with a list of available URIs. The GridLayout is
    filled with buttons that represent application aliases.

    Args:
        resolver (hab.Resolver): The resolver to change verbosity settings on.
        uri (str, optional): If passed, use this as the current uri. Otherwise
            the value stored in the users prefs is used.
        verbosity (int): Change the verbosity setting to this value. If None is passed,
            all results are be shown without any filtering.
        button_wrap_length (int) Indicates the number of buttons per column/row.
        button_layout (int) Sets the button layout to be either a horizontal focus
            or a vertical focus.
        parent (Qt.QtWidgets.QWidget, optional): Define a parent for this widget.
    """

    def __init__(
        self,
        resolver,
        uri=None,
        verbosity=0,
        button_wrap_length=3,
        button_layout=0,
        parent=None,
    ):
        super().__init__(parent)
        self.resolver = resolver
        self.verbosity = verbosity
        self.button_wrap_length = button_wrap_length
        self.button_layout = button_layout

        self.process_entry_points()
        self.init_gui(uri)

        # Window properties
        self.setWindowTitle("Hab Launch Aliases")
        self.setFixedWidth(400)
        self.center_window_position()

        # Create a auto-refresh timer by default that forces a refresh of hab.
        # This can be disabled by setting the site config setting to an empty string.
        self.refresh_timer = QtCore.QTimer(self)
        refresh_time = self.resolver.site.get("hab_gui_refresh_inverval", ["00:30:00"])
        refresh_time = refresh_time[0]
        if refresh_time:
            self.refresh_timer.timeout.connect(self.refresh_cache)
            refresh_time = utils.interval(refresh_time)
            logger.debug(f"Setting auto-refresh interval to {refresh_time} seconds")
            self.refresh_timer.start(refresh_time * 1000)

    def apply_layout(self):
        """Configures the layout of all widgets in this interface."""
        column_uri_widget = 1 if self.prefs_enabled else 0
        self.setCentralWidget(self.main_widget)
        self.layout.addWidget(self.uri_widget, 0, column_uri_widget)
        self.layout.addWidget(self.menu_button, 0, column_uri_widget + 1)
        self.layout.addWidget(self.alias_buttons, 1, 0, 1, -1)
        self.main_widget.setLayout(self.layout)

        # Ensure the tab order is intuitive. This doesn't come for free because
        # we need to pass uri_widget as an argument to pinned_uris which requires
        # creating it first and that breaks the default tab ordering.
        if self.prefs_enabled:
            self.setTabOrder(self.pinned_uris, self.uri_widget)
            self.setTabOrder(self.uri_widget, self.menu_button)
            self.setTabOrder(self.menu_button, self.alias_buttons)
            self.setTabOrder(self.alias_buttons, self.pinned_uris)

    def closeEvent(self, event):  # noqa: N802
        """Saves the currently selected URI on close if prefs are enabled."""
        self.resolver.user_prefs().uri = self.uri_widget.uri()
        super().closeEvent(event)

    def load_entry_point(self, name, default, allow_none=False):
        """Work function that loads the requested entry_point."""

        default = {"default": default}
        eps = self.resolver.site.entry_points_for_group(name, default=default)
        if not eps:
            raise ValueError(f"A valid entry_point for {name} must be defined")
        if allow_none and eps[0].value is None:
            return None
        return eps[0].load()

    def process_entry_points(self):
        """Loads the classes defined by the site entry_point system.
        These are later initialized by init_gui to create the UI.
        """
        # Used to launch a specific alias by `_cls_aliases_widget`
        self._cls_alias_widget = self.load_entry_point(
            "hab_gui_alias_widget", "hab_gui.widgets.alias_icon_button:AliasIconButton"
        )
        # Used to display alias launch widgets
        self._cls_aliases_widget = self.load_entry_point(
            "hab_gui_aliases_widget",
            "hab_gui.widgets.alias_button_grid:AliasButtonGrid",
        )
        # Allows the user to refresh hab configuration in case it has changed.
        self._cls_menu_button = self.load_entry_point(
            "hab_gui_menu_button",
            "hab_gui.widgets.menu_button:MenuButton",
        )
        # Allows the user to pin commonly used URI's
        self._cls_uri_pin_widget = self.load_entry_point(
            "hab_gui_uri_pin_widget",
            "hab_gui.widgets.pinned_uris_button:PinnedUriButton",
            allow_none=True,
        )
        # Interface the user uses to view and change the current URI.
        self._cls_uri_widget = self.load_entry_point(
            "hab_gui_uri_widget", "hab_gui.widgets.uri_combobox:URIComboBox"
        )

    def init_gui(self, uri=None):
        self.setWindowIcon(utils.Paths.icon("habihat.svg"))

        self.main_widget = QtWidgets.QWidget()
        self.layout = QtWidgets.QGridLayout()

        self.uri_widget = self._cls_uri_widget(
            self.resolver, verbosity=self.verbosity, parent=self
        )

        # If prefs are enabled, insert the Pinned URI widget
        self.prefs_enabled = self.resolver.user_prefs().enabled
        if self._cls_uri_pin_widget is None:
            # The pinning widget can be disabled by setting the entry point to null
            self.prefs_enabled = False

        # Create a refresh button
        self.menu_button = self._cls_menu_button(
            self.resolver, verbosity=self.verbosity, hab_widget=self
        )

        if self.prefs_enabled:
            self.pinned_uris = self._cls_uri_pin_widget(
                self.resolver, self.uri_widget, verbosity=self.verbosity, parent=self
            )
            self.layout.addWidget(self.pinned_uris, 0, 0)
            self.pinned_uris.uri_widget = self.uri_widget

        self.alias_buttons = self._cls_aliases_widget(
            self.resolver,
            self.button_wrap_length,
            self.button_layout,
            self.verbosity,
            button_cls=self._cls_alias_widget,
            parent=self,
        )

        self.apply_layout()

        # Connect signals
        self.uri_widget.uri_changed.connect(self.uri_changed)

        # Check for stored URI and apply it as the current text
        if uri is None:
            uri = str(self.resolver.user_prefs().uri_check())
        if uri:
            self.uri_widget.set_uri(uri)
            self.uri_changed(uri)

        # Ensure the URI widget has focus by default
        self.uri_widget.setFocus()

    @utils.cursor_override()
    def refresh_cache(self):
        logger.debug("Refresh cache")
        self.uri_widget.refresh()
        self.resolver.clear_caches()
        self.alias_buttons.refresh()

    def uri_changed(self, uri):
        self.alias_buttons.uri = uri
        self.alias_buttons.refresh()

    def center_window_position(self):
        # Place window onto screen center
        qt_rectangle = self.frameGeometry()
        app = QtWidgets.QApplication.instance()
        primary = app.primaryScreen()
        center_point = primary.availableGeometry().center()
        qt_rectangle.moveCenter(center_point)
        self.move(qt_rectangle.topLeft())


def main():
    app = QtWidgets.QApplication([])
    window = AliasLaunchWindow(hab.Resolver(target="hab-gui"), verbosity=1)
    window.show()
    app.exec_()



import datetime
import logging
import random
import time
from contextlib import contextmanager
from pathlib import Path

from Qt import QtCore, QtGui, QtWidgets

logger = logging.getLogger(__name__)


@contextmanager
def cursor_override(cursor=QtCore.Qt.BusyCursor):
    """Change the application cursor to wait while running the context/decorator.
    Ensures that the cursor is restored even if an exception is raised.
    """
    QtWidgets.QApplication.setOverrideCursor(cursor)
    try:
        yield
    finally:
        QtWidgets.QApplication.restoreOverrideCursor()


def entry_point_init(resolver, cmd, cli_args=None, **kwargs):
    """Used to apply startup configuration via site config.

    Example of site config replicating the default:
        {
            "append": {
                "entry_points": {
                    "hab_gui_init": {
                        "init": "hab_gui.entry_points.message_box:MessageBoxInit"
                    }
                }
            }
        }

    This uses the site entry_point `hab_gui_init` to initialize a class using
    the interface defined by `hab_gui.entry_points.BaseInit`. Defaults to
    `hab_gui.entry_points.message_box:MessageBoxInit` which shows a QMessageBox if
    any exceptions are raised. This prevents Qt from closing the application due
    to unhanded exceptions. If you want to disable this entry point default set
    the object reference to a empty string.

    Example of disabling entry point:
        {"append": {"entry_points": {"hab_gui_init": {"init": ""}}}}`
    """
    if cli_args is None:
        cli_args = {}

    default = {"init": "hab_gui.entry_points.message_box:MessageBoxInit"}

    # NOTE: kwargs should be added to allow for future changes to this call
    eps = resolver.site.entry_points_for_group("hab_gui_init", default=default)
    for ep in eps:
        if not ep.value:
            # Passing an empty value disables processing this entry point
            continue

        # Evaluate the entry point and initialize it
        func = ep.load()
        func(resolver, cmd, cli_args=cli_args, **kwargs)


def interval(interval, fmt="%H:%M:%S"):
    """Convert interval string to seconds. Uses the `%H:%M:%S` format by default.
    Source: https://stackoverflow.com/a/10663851
    """
    x = time.strptime(interval, fmt)
    return datetime.timedelta(
        hours=x.tm_hour, minutes=x.tm_min, seconds=x.tm_sec
    ).total_seconds()


def make_button_coords(button_list, wrap_length, arrangement):
    """Generates a dict that contains an alias_name key and a value that holds
    a 2D coordinate list.
    """
    array = dict()
    col = 0
    row = 0
    for i in range(0, len(button_list)):
        array[button_list[i]] = [row, col]
        if arrangement == 0:
            col += 1
            if col >= wrap_length:
                row += 1
                col = 0
        else:
            row += 1
            if row >= wrap_length:
                col += 1
                row = 0
    return array


class Paths:
    hab_gui = Path(__file__).parent
    resources = hab_gui / "resources"

    @classmethod
    def resource_path(cls, arg, *args):
        if isinstance(arg, str):
            return cls.resources.joinpath(arg, *args)
        return arg.joinpath(*args)

    @classmethod
    def icon(cls, arg, *args, **kwargs):
        path = cls.resource_path(arg, *args)
        return QtGui.QIcon(str(path), **kwargs)

    @classmethod
    def image(cls, arg, *args, **kwargs):
        path = cls.resource_path(arg, *args)
        return QtGui.QImage(str(path), **kwargs)


def get_splash_image(resolver):
    """
    Randomly grabs an image to use as a splash screen while starting Hab-Gui.

    A resolver object will be used to grab a site.json config that defines paths
    to any image or image directory.  Those will be distilled down to a list of
    valid images that can be used by the Hab-Gui SplashScreen class.  This method
    will then randomly choose an image from that list.
    """
    resolved_list = splash_paths(resolver)
    if not resolved_list:
        logger.debug("[Splash Image] No valid paths found to show.")
        return None
    return random.choice(resolved_list)


_splash_paths = None


def splash_paths(resolver, force=False):
    """Returns a list of all existing splash screen files found on disk.
    Uses the "splash_screen" list if defined in `resolver.site`. Any directories
    defined in this list will be expanded to include their direct children.
    Any file paths will also be included if they exist.

    The resolved list is cached, pass `force=True` to re-calculate the list.
    """
    global _splash_paths

    # Return the cached list if it was defined
    if not force and _splash_paths is not None:
        return _splash_paths

    paths = resolver.site.get("splash_screen", [])
    if not paths:
        # There are no paths to scan, just exit with an empty list
        _splash_paths = []
        return _splash_paths

    splash_paths = set()
    valid_extentions = set(
        [f".{x.data().decode()}" for x in QtGui.QImageReader.supportedImageFormats()]
    )

    for item in paths:
        item = Path(item)
        if item.is_dir():
            for scan_obj in item.iterdir():
                if scan_obj.is_file() and scan_obj.suffix in valid_extentions:
                    splash_paths.add(str(scan_obj))
        elif item.is_file() and item.suffix in valid_extentions:
            splash_paths.add(str(item))

    _splash_paths = list(splash_paths)
    return _splash_paths



import logging
import sys

from .base_init import BaseInit

logger = logging.getLogger(__name__)


class LoggingExceptionInit(BaseInit):
    """Overrides `sys.excepthook` and logs any exceptions raised instead of
    raising them. This prevents Qt from closing when an exception is raised.
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        sys.excepthook = self.excepthook

    def excepthook(self, cls, exception, tb):
        logger.exception("Captured Exception:", exc_info=(cls, exception, tb))



import logging
import traceback

from .logging_exception import LoggingExceptionInit

logger = logging.getLogger(__name__)


class MessageBoxInit(LoggingExceptionInit):
    """Overrides `sys.excepthook` and handles any exceptions raised instead of
    raising them. This prevents Qt from closing when an exception is raised.
    It logs the traceback and then shows a QMessageBox with the exception raised.
    """

    def excepthook(self, cls, exception, tb):
        super().excepthook(cls, exception, tb)

        from Qt.QtWidgets import QMessageBox

        # Show the user that an exception happened.
        msg = traceback.format_exception(cls, exception, tb)
        msg = "".join(msg)
        QMessageBox.critical(None, "Exception", msg)



import hab
from Qt import QtCore, QtWidgets


class URIComboBox(QtWidgets.QComboBox):
    """Create a QComboBox to store a given list of URIs.

    Args:
        resolver (hab.Resolver): The resolver to pull the URI data from Hab.
        verbosity (int): Pass along a verbosity value for filtering of URIs
        parent (Qt.QtWidgets.QWidget, optional): Define a parent for this widget.
    """

    uri_changed = QtCore.Signal(str)

    def __init__(self, resolver, verbosity=0, parent=None):
        super().__init__(parent)
        self.resolver = resolver
        self.verbosity = verbosity
        self.setEditable(True)
        _translate = QtCore.QCoreApplication.translate
        self.setPlaceholderText(_translate("Launch_Aliases", "Select a URI..."))
        self.lineEdit().setPlaceholderText(self.placeholderText())
        self.refresh()

        self.currentTextChanged.connect(self._emit_uri_changed)

    def _emit_uri_changed(self):
        self.uri_changed.emit(self.uri())

    def refresh(self):
        current = self.uri()
        self.clear()
        if self.uri is None:
            return
        with hab.utils.verbosity_filter(self.resolver, self.verbosity):
            items = self.resolver.dump_forest(self.resolver.configs, indent="")
            self.addItems(items)
        self.set_uri(current)

    def uri(self):
        return self.currentText()

    def set_uri(self, uri):
        # If the uri is already an item in the combo box, select it
        index = self.findText(uri)
        if index > -1:
            self.setCurrentIndex(index)
        else:
            # Otherwise update the text of the combo box to match
            self.setEditText(uri)



import logging

from Qt import QtCore, QtWidgets

logger = logging.getLogger(__name__)


class AliasButton(QtWidgets.QToolButton):
    """Create a QToolButton which will launch a specified alias via a subprocess.

    Args:
        cfg (hab.parsers.flat_config.FlatConfig): The config object which contains
        URI related data.
        alias_name (str): The alias name passed from AliasButtonGrid.
        parent (Qt.QtWidgets.QWidget, optional): Define a parent for this widget.
    """

    button_pressed = QtCore.Signal(str)

    def __init__(self, cfg, alias_name, parent=None):
        super().__init__(parent)
        self.cfg = cfg
        self.alias_name = alias_name

        self.alias_dict = self.cfg.aliases
        qsize_policy = QtWidgets.QSizePolicy
        size_policy = qsize_policy(qsize_policy.Minimum, qsize_policy.Preferred)
        self.setSizePolicy(size_policy)
        self.clicked.connect(self._button_action)
        self.refresh()

    def _button_action(self):
        """Launch the alias in a subprocess."""
        self.cfg.launch(self.alias_name)

    def refresh(self):
        alias = self.alias_dict[self.alias_name]
        label = alias.get("label", self.alias_name)
        self.setText(label)



from Qt import QtCore, QtWidgets


class URILineEdit(QtWidgets.QLineEdit):
    """Create a QLineEdit to store a given list of URIs.

    Args:
        resolver (hab.Resolver): The resolver to pull the URI data from Hab.
        verbosity (int): Pass along a verbosity value for filtering of URIs
        parent (Qt.QtWidgets.QWidget, optional): Define a parent for this widget.
    """

    uri_changed = QtCore.Signal(str)

    def __init__(self, resolver, verbosity=0, parent=None):
        super().__init__(parent)
        self.resolver = resolver
        self.verbosity = verbosity

        self.setPlaceholderText("Enter a URI...")
        self.textChanged.connect(self._emit_uri_changed)

    def _emit_uri_changed(self):
        self.uri_changed.emit(self.uri())

    def refresh(self):
        # Nothing to refresh on this widget
        pass

    def uri(self):
        return self.text()

    def set_uri(self, uri):
        self.setText(uri)
